package com.kingtom.office.controller;

import java.io.*;
import java.text.MessageFormat;
import java.util.UUID;

import javax.servlet.http.HttpServletResponse;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.io.FileUtil;
import com.kingtom.office.config.AppConfig;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.exec.CommandLine;
import org.apache.commons.exec.DefaultExecutor;
import org.apache.commons.exec.ExecuteException;
import org.apache.commons.exec.ExecuteResultHandler;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.multipart.MultipartFile;

/**
 * 应用模块名称<p>
 * 代码描述<p>
 * Copyright: Copyright (C) 2024 KingTom
 * , Inc. All rights reserved. <p>
 * Company: 成都诚唐科技有限责任公司<p>
 *
 * @author wany
 * @since 2024/8/5
 */
@Controller
@RequestMapping("office")
@Slf4j
public class OfficeController{

    @Autowired
    private AppConfig appConfig;

    private static final String FORMAT =
            File.separator + "yyyy" + File.separator + "MM" + File.separator + "dd" + File.separator;

    @PostMapping("toPdf")
    public void toPdf(@RequestParam("file") MultipartFile file, @RequestParam("fileType") String fileType,
            HttpServletResponse response) throws Exception{
        try{
            String uuid = StringUtils.replace(UUID.randomUUID().toString(), "-", "");
            File srcFile = new File(getFilePath() + uuid + "." + fileType);
            FileUtil.copyFile(file.getInputStream(), srcFile);
            File targetFile = targetFile = libreOfficeConvert(srcFile);
            //File targetFile = asposeConvert(srcFile);
            output(response, targetFile.getName(), targetFile);
        } catch(Exception e){
            log.error(e.getMessage(), e);
            throw new Exception("转换失败！");
        }
    }

    private static void output(HttpServletResponse response, String fileName, File file) throws IOException{
        try{
            response.setContentType("application/pdf");
            response.setHeader("Content-Disposition", "attachment;filename=\"" + fileName + "\"");
            FileInputStream fis = new FileInputStream(file);
            OutputStream out = response.getOutputStream();
            byte[] buffer = new byte[4096];
            int bytesRead;
            while((bytesRead = fis.read(buffer)) != -1){
                out.write(buffer, 0, bytesRead);
            }
            fis.close();
            out.flush();
            out.close();
        } catch(Exception e){
            response.setContentType("application/json;charset=UTF-8");
            PrintWriter writer = response.getWriter();
            String json = "{\"error\":\"" + e.getMessage() + "\"}";
            writer.print(json);
            writer.flush();
            writer.close();
        }
    }

    private String getFilePath(){
        return appConfig.getFileHome() + DateUtil.format(DateUtil.date(), FORMAT);
    }

    private synchronized File libreOfficeConvert(File officeFile) throws IOException, InterruptedException{
        DefaultExecutor exec = new DefaultExecutor();
        ExecuteResultHandler erh = new ExecuteResultHandler(){
            @Override
            public void onProcessComplete(int exitValue){
                log.info(" 转换完成! Exit value: {}", exitValue);
            }

            @Override
            public void onProcessFailed(ExecuteException e){
                log.error(" 转换失败!", e);
            }
        };
        File tempFolder = new File(getFilePath());
        String osName = System.getProperty("os.name");
        String command = null;
        if(osName.contains("Windows")){
            command = "cmd /c start soffice --headless --invisible --convert-to pdf:writer_pdf_Export \"{0}\" "
                    + "--outdir \"{1}\"";
        } else{
            String fileType = officeFile.getName().substring(officeFile.getName().lastIndexOf("."));
            String cmd;
            switch(fileType){
                case ".docx":
                case ".doc":
                    cmd = "writer";
                    break;
                case ".ppt":
                case ".pptx":
                    cmd = "impress";
                    break;
                case ".xls":
                case ".xlsx":
                    cmd = "calc";
                    break;
                default:
                    throw new RuntimeException("不支持的文件类型");
            }
            command = "libreoffice --headless --invisible --convert-to "
                    + "pdf:writer_pdf_Export \"{0}\" --outdir \"{1}\"";
        }
        command = MessageFormat.format(command, officeFile.getAbsolutePath(), tempFolder.getAbsolutePath());
        log.info("执行office文件转换任务，命令为{}", command);
        exec.execute(CommandLine.parse(command), erh);
        File outputFile = new File(tempFolder.getAbsolutePath() + File.separator + officeFile.getName()
                .substring(0, officeFile.getName().indexOf(".")) + ".pdf");
        //CountDownLatch latch = new CountDownLatch(1);不生效，所以换成了循环睡眠
        int time = 0;
        while((!outputFile.exists()) && time < appConfig.getWaitTime()){
            Thread.sleep(1000);
            time++;
        }
        return outputFile;
    }
}
